@@ -28,11 +28,39 @@
 
 static GHashTable* pattern;
 static gchar* blockcss = NULL;
+static gchar* blockcssprivate = NULL;
 static gchar* blockscript = NULL;
 
 static void
 adblock_parse_file (gchar* path);
 
+static gchar*
+adblock_build_js (const gchar* style,
+                  const gchar* private)
+{
+    return g_strdup_printf (
+        "window.addEventListener ('DOMContentLoaded',"
+        "function () {"
+        "var URL = location.href;"
+        "var sites = new Array(); %s;"
+        "var public = '%s';"
+        "for (var i in sites) {"
+        "if (URL.indexOf(i) != -1) {"
+        "public += sites[i];"
+        "break;"
+        "}}"
+        "public += ' {display: none !important;}';"
+        "var mystyle = document.createElement(\"style\");"
+        "mystyle.setAttribute(\"type\", \"text/css\");"
+        "mystyle.appendChild(document.createTextNode(public));"
+        "var head = document.getElementsByTagName(\"head\")[0];"
+        "if (head) head.appendChild(mystyle);"
+        "else document.documentElement.insertBefore(mystyle, document.documentElement.firstChild);"
+        "}, true);",
+        private,
+        style);
+}
+
 static gchar *
 adblock_fixup_regexp (gchar* src)
 {
@@ -102,7 +130,8 @@ adblock_reload_rules (MidoriExtension* extension)
     pattern = g_hash_table_new_full (g_str_hash, g_str_equal,
                    (GDestroyNotify)g_free,
                    (GDestroyNotify)g_regex_unref);
-    katze_assign (blockcss, NULL);
+    katze_assign (blockcss, g_strdup ("z-non-exist"));
+    katze_assign (blockcssprivate, g_strdup (""));
 
     while (filters[i++] != NULL)
     {
@@ -130,6 +159,7 @@ adblock_reload_rules (MidoriExtension* extension)
         }
         g_free (filename);
     }
+    katze_assign (blockscript, adblock_build_js (blockcss, blockcssprivate));
     g_strfreev (filters);
     g_free (folder);
 }
@@ -502,15 +532,25 @@ adblock_resource_request_starting_cb (WebKitWebView*         web_view,
                                       WebKitWebResource*     web_resource,
                                       WebKitNetworkRequest*  request,
                                       WebKitNetworkResponse* response,
-                                      MidoriView*            view)
+                                      GtkWidget*             image)
 {
     const gchar* uri = webkit_network_request_get_uri (request);
     if (!strncmp (uri, "data", 4))
         return;
     if (g_hash_table_find (pattern, (GHRFunc) adblock_is_matched, (char*)uri))
     {
+        #if 0
+        gchar* text;
+
+        if (gtk_widget_get_has_tooltip (image))
+           text = g_strdup_printf ("%s\n%s", gtk_widget_get_tooltip_text (image), uri);
+        else
+           text = g_strdup_printf ("Blocked content: \n%s", uri);
+        gtk_widget_set_tooltip_text (image, text);
+        g_free (text);
+        #endif
+
         webkit_network_request_set_uri (request, "about:blank");
-        /* TODO: Need to figure out how to indicate what was blocked */
     }
 }
 #else
@@ -522,6 +562,8 @@ adblock_session_request_queued_cb (SoupSession* session,
     gchar* uri = soup_uri ? soup_uri_to_string (soup_uri, FALSE) : g_strdup ("");
     if (g_hash_table_find (pattern, (GHRFunc) adblock_is_matched, uri))
     {
+        /* FIXME: Update image tooltip */
+
         soup_uri = soup_uri_new ("http://.invalid");
         soup_message_set_uri (msg, soup_uri);
         soup_uri_free (soup_uri);
@@ -530,22 +572,6 @@ adblock_session_request_queued_cb (SoupSession* session,
 }
 #endif
 
-static gchar*
-adblock_build_js (gchar* style)
-{
-    return g_strdup_printf (
-        "window.addEventListener ('DOMContentLoaded',"
-        "function () {"
-        "var mystyle = document.createElement(\"style\");"
-        "mystyle.setAttribute(\"type\", \"text/css\");"
-        "mystyle.appendChild(document.createTextNode('%s'));"
-        "var head = document.getElementsByTagName(\"head\")[0];"
-        "if (head) head.appendChild(mystyle);"
-        "else document.documentElement.insertBefore(mystyle, document.documentElement.firstChild);"
-        "}, true);",
-        style);
-}
-
 static void
 adblock_window_object_cleared_cb (GtkWidget*      web_view,
                                   WebKitWebFrame* web_frame,
@@ -557,26 +583,28 @@ adblock_window_object_cleared_cb (GtkWidget*      web_view,
 
 static void
 adblock_add_tab_cb (MidoriBrowser* browser,
-                    MidoriView*    view)
+                    MidoriView*    view,
+                    GtkWidget*     image)
 {
     GtkWidget* web_view = gtk_bin_get_child (GTK_BIN (view));
     g_signal_connect (web_view, "window-object-cleared",
         G_CALLBACK (adblock_window_object_cleared_cb), 0);
     #if HAVE_WEBKIT_RESOURCE_REQUEST
     g_signal_connect (web_view, "resource-request-starting",
-        G_CALLBACK (adblock_resource_request_starting_cb), view);
+        G_CALLBACK (adblock_resource_request_starting_cb), image);
     #endif
 }
 
 static void
 adblock_deactivate_cb (MidoriExtension* extension,
-                       MidoriBrowser*   browser);
+                       GtkWidget*       image);
 
 static void
 adblock_add_tab_foreach_cb (MidoriView*      view,
-                            MidoriBrowser*   browser)
+                            MidoriBrowser*   browser,
+                            GtkWidget*       image)
 {
-    adblock_add_tab_cb (browser, view);
+    adblock_add_tab_cb (browser, view, NULL);
 }
 
 static void
@@ -584,13 +612,27 @@ adblock_app_add_browser_cb (MidoriApp*       app,
                             MidoriBrowser*   browser,
                             MidoriExtension* extension)
 {
+    GtkWidget* statusbar;
+    GtkWidget* image;
+
+    statusbar = katze_object_get_object (browser, "statusbar");
+    #if 0
+    image = gtk_image_new_from_stock (STOCK_IMAGE, GTK_ICON_SIZE_MENU);
+    gtk_widget_show (image);
+    gtk_box_pack_start (GTK_BOX (statusbar), image, FALSE, FALSE, 3);
+    #else
+    image = NULL;
+    #endif
+
     midori_browser_foreach (browser,
-          (GtkCallback)adblock_add_tab_foreach_cb, browser);
-    g_signal_connect (browser, "add-tab", G_CALLBACK (adblock_add_tab_cb), 0);
+          (GtkCallback)adblock_add_tab_foreach_cb, image);
+    g_signal_connect (browser, "add-tab",
+        G_CALLBACK (adblock_add_tab_cb), image);
     g_signal_connect (browser, "populate-tool-menu",
         G_CALLBACK (adblock_browser_populate_tool_menu_cb), extension);
     g_signal_connect (extension, "deactivate",
-        G_CALLBACK (adblock_deactivate_cb), browser);
+        G_CALLBACK (adblock_deactivate_cb), image);
+    g_object_unref (statusbar);
 }
 
 static void
@@ -600,11 +642,39 @@ adblock_frame_add (gchar* line)
 
     (void)*line++;
     (void)*line++;
-    new_blockcss = g_strdup_printf ("%s %s { display: none !important; }",
-                                    blockcss, line);
+    new_blockcss = g_strdup_printf ("%s, %s", blockcss, line);
     katze_assign (blockcss, new_blockcss);
 }
 
+static void
+adblock_frame_add_private (gchar* line)
+{
+    gchar* new_blockcss;
+    gchar** data;
+    data = g_strsplit (line, "##", 2);
+
+    if (strstr (data[0],","))
+    {
+        gchar** domains;
+        gint max, i;
+        domains = g_strsplit (data[0], ",", -1);
+        for (max = i = 0; domains [i]; i++)
+        {
+            new_blockcss = g_strdup_printf ("%s;\nsites['%s']+=',%s'",
+                blockcssprivate, g_strstrip (domains[i]), data[1]);
+            katze_assign (blockcssprivate, new_blockcss);
+        }
+        g_strfreev (domains);
+    }
+    else
+    {
+        new_blockcss = g_strdup_printf ("%s;\nsites['%s']+=',%s'",
+            blockcssprivate, data[0], data[1]);
+        katze_assign (blockcssprivate, new_blockcss);
+    }
+    g_strfreev (data);
+}
+
 static gchar*
 adblock_parse_line (gchar* line)
 {
@@ -624,7 +694,8 @@ adblock_parse_line (gchar* line)
     if (strstr (line,"$"))
         return NULL;
     /* Got block hider */
-    if (line[0] == '#' && line[1] == '#' && (line[2] == '.'||line[2] == '#'||line[2] == 'a'))
+    if (line[0] == '#' && line[1] == '#' && (line[2] == '.' || line[2] == '#'
+     || line[2] == 'A' || line[2] == 'a' || line[2] == 'D' || line[2] == 'U'))
     {
         adblock_frame_add (line);
         return NULL;
@@ -632,9 +703,12 @@ adblock_parse_line (gchar* line)
     /* FIXME: Do we have smth else starting with ##? */
     if (line[0] == '#' && line[1] == '#')
         return NULL;
-    /* FIXME: No support for per domain element hiding */
+
     if (strstr (line,"##"))
+    {
+        adblock_frame_add_private (line);
         return NULL;
+    }
     /* FIXME: No support for [include] and [exclude] tags */
     if (line[0] == '[')
         return NULL;
@@ -670,33 +744,36 @@ adblock_parse_file (gchar* path)
             else
                 g_hash_table_insert (pattern, parsed, regex);
         }
-        katze_assign (blockscript, adblock_build_js (blockcss));
         fclose (file);
     }
 }
 
 static void
-adblock_deactivate_tabs (MidoriView*      view,
-                         MidoriBrowser*   browser)
+adblock_deactivate_tabs (MidoriView* view,
+                         GtkWidget*  image)
 {
     GtkWidget* web_view = gtk_bin_get_child (GTK_BIN (view));
+    MidoriBrowser* browser = midori_browser_get_for_widget (image);
+
     g_signal_handlers_disconnect_by_func (
        browser, adblock_add_tab_cb, 0);
     g_signal_handlers_disconnect_by_func (
        web_view, adblock_window_object_cleared_cb, 0);
     #if HAVE_WEBKIT_RESOURCE_REQUEST
     g_signal_handlers_disconnect_by_func (
-       web_view, adblock_resource_request_starting_cb, view);
+       web_view, adblock_resource_request_starting_cb, image);
     #else
     g_signal_handlers_disconnect_by_func (
        webkit_get_default_session (), adblock_session_request_queued_cb, NULL);
     #endif
+    gtk_widget_destroy (image);
 }
 
 static void
 adblock_deactivate_cb (MidoriExtension* extension,
-                       MidoriBrowser*   browser)
+                       GtkWidget*       image)
 {
+    MidoriBrowser* browser = midori_browser_get_for_widget (image);
     MidoriApp* app = midori_extension_get_app (extension);
 
     g_signal_handlers_disconnect_by_func (
@@ -705,9 +782,10 @@ adblock_deactivate_cb (MidoriExtension* extension,
         extension, adblock_deactivate_cb, browser);
     g_signal_handlers_disconnect_by_func (
         app, adblock_app_add_browser_cb, extension);
-    midori_browser_foreach (browser, (GtkCallback)adblock_deactivate_tabs, browser);
+    midori_browser_foreach (browser, (GtkCallback)adblock_deactivate_tabs, image);
 
     katze_assign (blockcss, NULL);
+    katze_assign (blockcssprivate, NULL);
     g_hash_table_destroy (pattern);
 }
 
