@@ -1,5 +1,5 @@
 /*
- Copyright (C) 2007-2008 Christian Dywan <christian@twotoasts.de>
+ Copyright (C) 2007-2009 Christian Dywan <christian@twotoasts.de>
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
@@ -11,15 +11,30 @@
 
 #include "katze-utils.h"
 
+#include <glib/gstdio.h>
 #include <glib/gi18n.h>
 
 #include <string.h>
 
+#if HAVE_CONFIG_H
+    #include "config.h"
+#endif
+
+#ifdef HAVE_HILDON_2_2
+    #include <hildon/hildon.h>
+#endif
+
 static void
 proxy_toggle_button_toggled_cb (GtkToggleButton* button,
                                 GObject*         object)
 {
-    gboolean toggled = gtk_toggle_button_get_active (button);
+    gboolean toggled;
+    #ifdef HAVE_HILDON_2_2
+    if (HILDON_IS_CHECK_BUTTON (button))
+        toggled = hildon_check_button_get_active (HILDON_CHECK_BUTTON (button));
+    #else
+    toggled = gtk_toggle_button_get_active (button);
+    #endif
     const gchar* property = g_object_get_data (G_OBJECT (button), "property");
     g_object_set (object, property, toggled, NULL);
 }
@@ -60,6 +75,79 @@ proxy_combo_box_text_changed_cb (GtkComboBox* button,
     g_object_set (object, property, text, NULL);
 }
 
+static const gchar*
+katze_app_info_get_commandline (GAppInfo* info)
+{
+    const gchar* exe;
+
+    #if GLIB_CHECK_VERSION (2, 20, 0)
+    exe = g_app_info_get_commandline (info);
+    #else
+    exe = g_object_get_data (G_OBJECT (info), "katze-cmdline");
+    #endif
+    if (!exe)
+        exe = g_app_info_get_executable (info);
+    return exe;
+}
+
+static gboolean
+proxy_entry_focus_out_event_cb (GtkEntry*      entry,
+                                GdkEventFocus* event,
+                                GObject*       object);
+
+static void
+proxy_combo_box_apps_changed_cb (GtkComboBox* button,
+                                 GObject*     object)
+{
+    guint active = gtk_combo_box_get_active (button);
+    GtkTreeModel* model = gtk_combo_box_get_model (button);
+    GtkTreeIter iter;
+
+    if (gtk_tree_model_iter_nth_child (model, &iter, NULL, active))
+    {
+        GAppInfo* info;
+        gboolean use_entry;
+        GtkWidget* child;
+        const gchar* exe;
+        const gchar* property = g_object_get_data (G_OBJECT (button), "property");
+
+        gtk_tree_model_get (model, &iter, 0, &info, -1);
+
+        use_entry = info && !g_app_info_get_icon (info);
+        child = gtk_bin_get_child (GTK_BIN (button));
+        if (use_entry && GTK_IS_CELL_VIEW (child))
+        {
+            GtkWidget* entry = gtk_entry_new ();
+            exe = g_app_info_get_executable (info);
+            if (exe && *exe && strcmp (exe, "%f"))
+                gtk_entry_set_text (GTK_ENTRY (entry), exe);
+            gtk_widget_show (entry);
+            gtk_container_add (GTK_CONTAINER (button), entry);
+            gtk_widget_grab_focus (entry);
+            g_signal_connect (entry, "focus-out-event",
+                G_CALLBACK (proxy_entry_focus_out_event_cb), object);
+            g_object_set_data_full (G_OBJECT (entry), "property",
+                                    g_strdup (property), g_free);
+        }
+        else if (!use_entry && GTK_IS_ENTRY (child))
+        {
+            /* Force the combo to change the item again */
+            gtk_widget_destroy (child);
+            gtk_combo_box_set_active (button, 0);
+            gtk_combo_box_set_active_iter (button, &iter);
+        }
+
+        if (info)
+        {
+            exe = katze_app_info_get_commandline (info);
+            g_object_set (object, property, exe, NULL);
+            g_object_unref (info);
+        }
+        else
+            g_object_set (object, property, "", NULL);
+    }
+}
+
 static void
 proxy_entry_activate_cb (GtkEntry* entry,
                          GObject*  object)
@@ -99,14 +187,73 @@ proxy_spin_button_changed_cb (GtkSpinButton* button,
     }
 }
 
+#ifdef HAVE_HILDON_2_2
+static void
+proxy_picker_button_changed_cb (HildonPickerButton* button,
+                                GObject*            object)
+{
+    gint value = hildon_picker_button_get_active (button);
+    const gchar* property = g_object_get_data (G_OBJECT (button), "property");
+    g_object_set (object, property, value, NULL);
+    /* FIXME: Implement custom-PROPERTY */
+}
+#else
 static void
 proxy_combo_box_changed_cb (GtkComboBox* button,
                             GObject*     object)
 {
     gint value = gtk_combo_box_get_active (button);
     const gchar* property = g_object_get_data (G_OBJECT (button), "property");
+    gint custom_value = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (button),
+                                         "katze-custom-value"));
+    const gchar* custom_property = g_object_get_data (G_OBJECT (button),
+                                                      "katze-custom-property");
+
+    if (custom_value)
+    {
+        GtkWidget* child = gtk_bin_get_child (GTK_BIN (button));
+
+        if (value == custom_value && GTK_IS_CELL_VIEW (child))
+        {
+            GtkWidget* entry = gtk_entry_new ();
+            gchar* custom_text = katze_object_get_string (object, custom_property);
+            if (custom_text && *custom_text)
+                gtk_entry_set_text (GTK_ENTRY (entry), custom_text);
+            g_free (custom_text);
+            gtk_widget_show (entry);
+            gtk_container_add (GTK_CONTAINER (button), entry);
+            gtk_widget_grab_focus (entry);
+            g_signal_connect (entry, "focus-out-event",
+                G_CALLBACK (proxy_entry_focus_out_event_cb), object);
+            g_object_set_data_full (G_OBJECT (entry), "property",
+                                    g_strdup (custom_property), g_free);
+        }
+        else if (value != custom_value && GTK_IS_ENTRY (child))
+        {
+            /* Force the combo to change the item again */
+            gtk_widget_destroy (child);
+            gtk_combo_box_set_active (button, value + 1);
+            gtk_combo_box_set_active (button, value);
+        }
+    }
+
     g_object_set (object, property, value, NULL);
+
+    if (custom_value)
+    {
+        #if GTK_CHECK_VERSION (2, 12, 0)
+        if (value == custom_value)
+            gtk_widget_set_tooltip_text (GTK_WIDGET (button), NULL);
+        else
+        {
+            gchar* custom_text = katze_object_get_string (object, custom_property);
+            gtk_widget_set_tooltip_text (GTK_WIDGET (button), custom_text);
+            g_free (custom_text);
+        }
+        #endif
+    }
 }
+#endif
 
 static void
 proxy_object_notify_boolean_cb (GObject*    object,
@@ -145,6 +292,38 @@ proxy_widget_string_destroy_cb (GtkWidget* proxy,
         proxy_object_notify_string_cb, proxy);
 }
 
+static GList*
+katze_app_info_get_all_for_category (const gchar* category)
+{
+    GList* all_apps = g_app_info_get_all ();
+    GList* apps = NULL;
+    guint i = 0;
+    GAppInfo* info;
+    while ((info = g_list_nth_data (all_apps, i++)))
+    {
+        #ifdef GDK_WINDOWING_X11
+        gchar* filename = g_strconcat ("applications/", g_app_info_get_id (info), NULL);
+        GKeyFile* file = g_key_file_new ();
+
+        if (g_key_file_load_from_data_dirs (file, filename, NULL, G_KEY_FILE_NONE, NULL))
+        {
+            gchar* cat = g_key_file_get_string (file, "Desktop Entry",
+                                                "Categories", NULL);
+            if (cat && g_strrstr (cat, category))
+                apps = g_list_append (apps, info);
+
+            g_free (cat);
+        }
+        g_key_file_free (file);
+        g_free (filename);
+        #else
+        apps = g_list_append (apps, info);
+        #endif
+    }
+    g_list_free (all_apps);
+    return apps;
+}
+
 /**
  * katze_property_proxy:
  * @object: a #GObject
@@ -173,11 +352,22 @@ proxy_widget_string_destroy_cb (GtkWidget* proxy,
  *     Since 0.2.0 the following hints are also supported:
  *     "font-monospace": the widget created will be particularly suitable for
  *         choosing a fixed-width font from installed fonts.
+ *     Since 0.2.1 the following hints are also supported:
+ *     "application-TYPE": the widget created will be particularly suitable
+ *         for choosing an application to open TYPE files, ie. "text/plain".
+ *     "application-CATEGORY": the widget created will be particularly suitable
+ *         for choosing an application to open CATEGORY files, ie. "Network".
+ *     "custom-PROPERTY": the last value of an enumeration will be the "custom"
+ *         value, where the user may enter text freely, which then updates
+ *         the property PROPERTY instead. This applies only to enumerations.
  *
  * Any other values for @hint are silently ignored.
  *
  * Since 0.1.2 strings without hints and booleans are truly synchronous
- * including property changes causing the proxy to be updated.
+ *     including property changes causing the proxy to be updated.
+ *
+ * Since 0.2.1 the proxy may contain a label if the platform
+ *     has according widgets.
  *
  * Return value: a new widget
  **/
@@ -216,12 +406,23 @@ katze_property_proxy (gpointer     object,
         gchar* notify_property;
         gboolean toggled = katze_object_get_boolean (object, property);
 
-        widget = gtk_check_button_new ();
-        if (_hint == g_intern_string ("toggle"))
-            gtk_toggle_button_set_mode (GTK_TOGGLE_BUTTON (widget), FALSE);
-        else
+        #ifdef HAVE_HILDON_2_2
+        if (_hint != g_intern_string ("toggle"))
+        {
+            widget = hildon_check_button_new (HILDON_SIZE_FINGER_HEIGHT | HILDON_SIZE_AUTO_WIDTH);
             gtk_button_set_label (GTK_BUTTON (widget), gettext (nick));
-        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), toggled);
+            hildon_check_button_set_active (HILDON_CHECK_BUTTON (widget), toggled);
+        }
+        else
+        #endif
+        {
+            widget = gtk_check_button_new ();
+            if (_hint == g_intern_string ("toggle"))
+                gtk_toggle_button_set_mode (GTK_TOGGLE_BUTTON (widget), FALSE);
+            else
+                gtk_button_set_label (GTK_BUTTON (widget), gettext (nick));
+            gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), toggled);
+        }
         g_signal_connect (widget, "toggled",
                           G_CALLBACK (proxy_toggle_button_toggled_cb), object);
         notify_property = g_strdup_printf ("notify::%s", property);
@@ -315,6 +516,93 @@ katze_property_proxy (gpointer     object,
                           G_CALLBACK (proxy_combo_box_text_changed_cb), object);
         g_free (families);
     }
+    else if (type == G_TYPE_PARAM_STRING && hint && g_str_has_prefix (hint, "application-"))
+    {
+        GtkListStore* model;
+        GtkCellRenderer* renderer;
+        GtkComboBox* combo;
+        GList* apps;
+        const gchar* app_type = &hint[12];
+
+        model = gtk_list_store_new (3, G_TYPE_APP_INFO, G_TYPE_STRING, G_TYPE_STRING);
+        widget = gtk_combo_box_new_with_model (GTK_TREE_MODEL (model));
+        renderer = gtk_cell_renderer_pixbuf_new ();
+        gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (widget), renderer, FALSE);
+        gtk_cell_layout_add_attribute (GTK_CELL_LAYOUT (widget), renderer, "icon-name", 1);
+        renderer = gtk_cell_renderer_text_new ();
+        gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (widget), renderer, TRUE);
+        gtk_cell_layout_add_attribute (GTK_CELL_LAYOUT (widget), renderer, "text", 2);
+        combo = GTK_COMBO_BOX (widget);
+        apps = g_app_info_get_all_for_type (app_type);
+        if (!apps)
+            apps = katze_app_info_get_all_for_category (app_type);
+
+        string = katze_object_get_string (object, property);
+        if (!g_strcmp0 (string, ""))
+            katze_assign (string, NULL);
+
+        if (apps)
+        {
+            GtkTreeIter iter_none;
+            gint i = 0;
+            GAppInfo* info;
+
+            gtk_list_store_insert_with_values (model, &iter_none, 0,
+                0, NULL, 1, NULL, 2, _("None"), -1);
+
+            while ((info = g_list_nth_data (apps, i++)))
+            {
+                const gchar* name = g_app_info_get_name (info);
+                GIcon* icon = g_app_info_get_icon (info);
+                gchar* icon_name = g_icon_to_string (icon);
+                GtkTreeIter iter;
+
+                gtk_list_store_insert_with_values (model, &iter, G_MAXINT,
+                    0, info, 1, icon_name, 2, name, -1);
+                if (string && !strcmp (katze_app_info_get_commandline (info), string))
+                    gtk_combo_box_set_active_iter (combo, &iter);
+
+                g_free (icon_name);
+            }
+
+            info = g_app_info_create_from_commandline ("",
+                "", G_APP_INFO_CREATE_NONE, NULL);
+            gtk_list_store_insert_with_values (model, NULL, G_MAXINT,
+                0, info, 1, NULL, 2, _("Custom..."), -1);
+            g_object_unref (info);
+
+            if (gtk_combo_box_get_active (combo) == -1)
+            {
+                if (string)
+                {
+                    GtkWidget* entry;
+                    const gchar* exe;
+
+                    info = g_app_info_create_from_commandline (string,
+                        NULL, G_APP_INFO_CREATE_NONE, NULL);
+                    #if !GLIB_CHECK_VERSION (2, 20, 0)
+                    g_object_set_data (G_OBJECT (info), "katze-cmdline", string);
+                    #endif
+                    entry = gtk_entry_new ();
+                    exe = g_app_info_get_executable (info);
+                    if (exe && *exe && strcmp (exe, "%f"))
+                        gtk_entry_set_text (GTK_ENTRY (entry), string);
+                    gtk_widget_show (entry);
+                    gtk_container_add (GTK_CONTAINER (combo), entry);
+                    g_object_unref (info);
+                    g_signal_connect (entry, "focus-out-event",
+                        G_CALLBACK (proxy_entry_focus_out_event_cb), object);
+                    g_object_set_data_full (G_OBJECT (entry), "property",
+                                            g_strdup (property), g_free);
+                }
+                else
+                    gtk_combo_box_set_active_iter (combo, &iter_none);
+            }
+        }
+        g_list_free (apps);
+        g_signal_connect (widget, "changed",
+                          G_CALLBACK (proxy_combo_box_apps_changed_cb), object);
+    }
     else if (type == G_TYPE_PARAM_STRING)
     {
         gchar* notify_property;
@@ -368,23 +656,81 @@ katze_property_proxy (gpointer     object,
         GEnumClass* enum_class = G_ENUM_CLASS (
             g_type_class_ref (pspec->value_type));
         gint value = katze_object_get_enum (object, property);
+        const gchar* custom = NULL;
 
+        if (hint && g_str_has_prefix (hint, "custom-"))
+            custom = &hint[7];
+
+        #ifdef HAVE_HILDON_2_2
+        GtkWidget* selector;
+
+        widget = hildon_picker_button_new (
+            HILDON_SIZE_FINGER_HEIGHT | HILDON_SIZE_AUTO_WIDTH,
+            HILDON_BUTTON_ARRANGEMENT_HORIZONTAL);
+        selector = hildon_touch_selector_new_text ();
+        hildon_button_set_title (HILDON_BUTTON (widget), gettext (nick));
+        hildon_picker_button_set_selector (HILDON_PICKER_BUTTON (widget),
+                                           HILDON_TOUCH_SELECTOR (selector));
+        #else
         widget = gtk_combo_box_new_text ();
+        #endif
         for (i = 0; i < enum_class->n_values; i++)
         {
             const gchar* label = gettext (enum_class->values[i].value_nick);
+            #ifdef HAVE_HILDON_2_2
+            hildon_touch_selector_append_text (HILDON_TOUCH_SELECTOR (selector), label);
+            #else
             gtk_combo_box_append_text (GTK_COMBO_BOX (widget), label);
+            #endif
         }
+        #ifdef HAVE_HILDON_2_2
+        hildon_touch_selector_set_active (HILDON_TOUCH_SELECTOR (selector), 0, value);
+        g_signal_connect (widget, "value-changed",
+                          G_CALLBACK (proxy_picker_button_changed_cb), object);
+        #else
         gtk_combo_box_set_active (GTK_COMBO_BOX (widget), value);
         g_signal_connect (widget, "changed",
                           G_CALLBACK (proxy_combo_box_changed_cb), object);
+        #endif
+        if (custom)
+        {
+            gchar* custom_text = katze_object_get_string (object, custom);
+
+            if (value == (gint)(enum_class->n_values - 1))
+            {
+                GtkWidget* entry = gtk_entry_new ();
+                gchar* text = katze_object_get_string (object, custom);
+                if (text && *text)
+                    gtk_entry_set_text (GTK_ENTRY (entry), text);
+                gtk_widget_show (entry);
+                gtk_container_add (GTK_CONTAINER (widget), entry);
+                g_signal_connect (entry, "focus-out-event",
+                    G_CALLBACK (proxy_entry_focus_out_event_cb), object);
+                g_object_set_data_full (G_OBJECT (entry), "property",
+                                        g_strdup (custom), g_free);
+            }
+            #if GTK_CHECK_VERSION (2, 12, 0)
+            else
+                gtk_widget_set_tooltip_text (widget, custom_text);
+            #endif
+
+            g_free (custom_text);
+
+            g_object_set_data (G_OBJECT (widget), "katze-custom-value",
+                               GINT_TO_POINTER (enum_class->n_values - 1));
+            g_object_set_data (G_OBJECT (widget), "katze-custom-property",
+                               (gpointer)custom);
+        }
         g_type_class_unref (enum_class);
     }
     else
         widget = gtk_label_new (gettext (nick));
     g_free (string);
 
-    gtk_widget_set_tooltip_text (widget, g_param_spec_get_blurb (pspec));
+    #if GTK_CHECK_VERSION (2, 12, 0)
+    if (!gtk_widget_get_tooltip_text (widget))
+        gtk_widget_set_tooltip_text (widget, g_param_spec_get_blurb (pspec));
+    #endif
     gtk_widget_set_sensitive (widget, pspec->flags & G_PARAM_WRITABLE);
 
     g_object_set_data_full (G_OBJECT (widget), "property",
@@ -401,6 +747,9 @@ katze_property_proxy (gpointer     object,
  * Create a label widget displaying the name of the specified object's property.
  *
  * Return value: a new label widget
+ *
+ * Since 0.2.1 the label will be empty if the property proxy for the
+ *    same property would contain a label already.
  **/
 GtkWidget*
 katze_property_label (gpointer     object,
@@ -421,8 +770,16 @@ katze_property_label (gpointer     object,
                    property, G_OBJECT_CLASS_NAME (class));
         return gtk_label_new (property);
     }
+
+    #ifdef HAVE_HILDON_2_2
+    if (G_PARAM_SPEC_TYPE (pspec) == G_TYPE_PARAM_ENUM)
+        return gtk_label_new (NULL);
+    #endif
+
     nick = g_param_spec_get_nick (pspec);
     widget = gtk_label_new (nick);
+    gtk_widget_set_tooltip_text (widget, g_param_spec_get_blurb (pspec));
+    gtk_misc_set_alignment (GTK_MISC (widget), 0.0, 0.5);
 
     return widget;
 }
@@ -849,3 +1206,109 @@ katze_object_get_object (gpointer     object,
     g_object_get (object, property, &value, NULL);
     return value;
 }
+
+/**
+ * katze_mkdir_with_parents:
+ * @pathname: a pathname in the GLib file name encoding
+ * @mode: permissions to use for newly created directories
+ *
+ * Create a directory if it doesn't already exist. Create intermediate
+ * parent directories as needed, too.
+ *
+ * Similar to g_mkdir_with_parents() but returning early if the
+ * @pathname refers to an existing directory.
+ *
+ * Returns: 0 if the directory already exists, or was successfully
+ * created. Returns -1 if an error occurred, with errno set.
+ *
+ * Since: 0.2.1
+ */
+/* Creating directories recursively
+   Copyright 2000 Red Hat, Inc.
+   Originally copied from Glib 2.20, coding style adjusted
+   Modified to determine file existence early and pathname must be != NULL */
+int
+katze_mkdir_with_parents (const gchar* pathname,
+                          int          mode)
+{
+  gchar* fn, *p;
+
+  if (g_file_test (pathname, G_FILE_TEST_EXISTS))
+      return 0;
+
+  fn = g_strdup (pathname);
+
+  if (g_path_is_absolute (fn))
+    p = (gchar *) g_path_skip_root (fn);
+  else
+    p = fn;
+
+  do
+  {
+      while (*p && !G_IS_DIR_SEPARATOR (*p))
+          p++;
+
+      if (!*p)
+          p = NULL;
+      else
+          *p = '\0';
+
+      if (!g_file_test (fn, G_FILE_TEST_EXISTS))
+      {
+          if (g_mkdir (fn, mode) == -1)
+          {
+              g_free (fn);
+              return -1;
+          }
+      }
+      else if (!g_file_test (fn, G_FILE_TEST_IS_DIR))
+      {
+          g_free (fn);
+          return -1;
+      }
+      if (p)
+      {
+          *p++ = G_DIR_SEPARATOR;
+          while (*p && G_IS_DIR_SEPARATOR (*p))
+              p++;
+      }
+  }
+  while (p);
+
+  g_free (fn);
+
+  return 0;
+}
+
+/**
+ * katze_widget_has_touchscreen_mode:
+ * @widget: a #GtkWidget, or %NULL
+ *
+ * Determines whether @widget should operate in touchscreen
+ * mode, as determined by GtkSettings or the environment
+ * variable MIDORI_TOUCHSCREEN.
+ *
+ * If @widget is %NULL, the default screen will be used.
+ *
+ * Returns: %TRUE if touchscreen mode should be used
+ *
+ * Since: 0.2.1
+ */
+gboolean
+katze_widget_has_touchscreen_mode (GtkWidget* widget)
+{
+    const gchar* touchscreen = g_getenv ("MIDORI_TOUCHSCREEN");
+    if (touchscreen && touchscreen[0] == '1')
+        return TRUE;
+    else if (touchscreen && touchscreen[0] == '0')
+        return FALSE;
+    else
+    {
+        GdkScreen* screen = widget && gtk_widget_has_screen (widget)
+            ? gtk_widget_get_screen (widget) : gdk_screen_get_default ();
+        GtkSettings* gtk_settings = gtk_settings_get_for_screen (screen);
+        gboolean enabled;
+        g_object_get (gtk_settings, "gtk-touchscreen-mode", &enabled, NULL);
+        return enabled;
+    }
+}
