@@ -50,8 +50,8 @@
     #include <locale.h>
 #endif
 
-#if HAVE_HILDON
-    #include <libosso.h>
+#ifdef HAVE_SIGNAL_H
+    #include <signal.h>
 #endif
 
 #define MIDORI_HISTORY_ERROR g_quark_from_string("MIDORI_HISTORY_ERROR")
@@ -67,7 +67,7 @@ static gchar*
 build_config_filename (const gchar* filename)
 {
     const gchar* path = sokoke_set_config_dir (NULL);
-    g_mkdir_with_parents (path, 0700);
+    katze_mkdir_with_parents (path, 0700);
     return g_build_filename (path, filename, NULL);
 }
 
@@ -966,8 +966,9 @@ midori_app_add_browser_cb (MidoriApp*     app,
     midori_panel_append_page (MIDORI_PANEL (panel), MIDORI_VIEWABLE (addon));
 
     /* Extensions */
-    addon = g_object_new (MIDORI_TYPE_EXTENSIONS, "app", app, NULL);
+    addon = g_object_new (MIDORI_TYPE_EXTENSIONS, NULL);
     gtk_widget_show (addon);
+    g_object_set (addon, "app", app, NULL);
     midori_panel_append_page (MIDORI_PANEL (panel), MIDORI_VIEWABLE (addon));
 
     g_object_unref (panel);
@@ -1419,7 +1420,7 @@ midori_load_session (gpointer data)
         GtkAction* action = gtk_action_group_get_action (action_group, "LastSession");
         g_signal_connect (action, "activate",
             G_CALLBACK (midori_browser_action_last_session_activate_cb), browser);
-        gtk_action_set_sensitive (action, TRUE);
+        gtk_action_set_visible (action, TRUE);
     }
     midori_app_add_browser (app, browser);
     gtk_widget_show (GTK_WIDGET (browser));
@@ -1575,6 +1576,19 @@ midori_remove_config_file (gint         clear_prefs,
     }
 }
 
+#ifdef HAVE_SIGNAL_H
+static void
+signal_handler (int signal_id)
+{
+    if (signal_id == SIGHUP || signal_id == SIGINT
+     || signal_id == SIGTERM || signal_id == SIGQUIT)
+    {
+        midori_app_quit_cb (NULL);
+        gtk_main_quit ();
+    }
+}
+#endif
+
 int
 main (int    argc,
       char** argv)
@@ -1593,8 +1607,10 @@ main (int    argc,
     {
        { "app", 'a', 0, G_OPTION_ARG_STRING, &webapp,
        N_("Run ADDRESS as a web application"), N_("ADDRESS") },
+       #if !HAVE_HILDON
        { "config", 'c', 0, G_OPTION_ARG_FILENAME, &config,
        N_("Use FOLDER as configuration folder"), N_("FOLDER") },
+       #endif
        { "run", 'r', 0, G_OPTION_ARG_NONE, &run,
        N_("Run the specified filename as javascript"), NULL },
        #if WEBKIT_CHECK_VERSION (1, 1, 6)
@@ -1627,9 +1643,6 @@ main (int    argc,
     sqlite3* db;
     gint max_history_age;
     #endif
-    #if HAVE_HILDON
-    osso_context_t* osso_context;
-    #endif
     gint clear_prefs = MIDORI_CLEAR_NONE;
 
     #if ENABLE_NLS
@@ -1650,6 +1663,13 @@ main (int    argc,
     textdomain (GETTEXT_PACKAGE);
     #endif
 
+    #ifdef HAVE_SIGNAL_H
+    signal (SIGHUP, &signal_handler);
+    signal (SIGINT, &signal_handler);
+    signal (SIGTERM, &signal_handler);
+    signal (SIGQUIT, &signal_handler);
+    #endif
+
     /* Parse cli options */
     webapp = NULL;
     config = NULL;
@@ -1752,16 +1772,6 @@ main (int    argc,
     if (run)
         return midori_run_script (uris ? *uris : NULL);
 
-    #if HAVE_HILDON
-    osso_context = osso_initialize (PACKAGE_NAME, PACKAGE_VERSION, FALSE, NULL);
-
-    if (!osso_context)
-    {
-        g_critical ("Error initializing OSSO D-Bus context - Midori");
-        return 1;
-    }
-    #endif
-
     if (config && !g_path_is_absolute (config))
     {
         g_critical (_("The specified configuration folder is invalid."));
@@ -1974,7 +1984,9 @@ main (int    argc,
         {
             item = katze_item_new ();
             /* Construct an absolute path if the file is relative */
-            if (g_file_test (uri, G_FILE_TEST_EXISTS | G_FILE_TEST_IS_REGULAR))
+            if (g_path_is_absolute (uri))
+                uri_ready = g_strconcat ("file://", uri, NULL);
+            else if (g_file_test (uri, G_FILE_TEST_EXISTS | G_FILE_TEST_IS_REGULAR))
             {
                 gchar* current_dir = g_get_current_dir ();
                 uri_ready = g_strconcat ("file://", current_dir,
@@ -2095,10 +2107,6 @@ main (int    argc,
 
     gtk_main ();
 
-    #if HAVE_HILDON
-    osso_deinitialize (osso_context);
-    #endif
-
     settings = katze_object_get_object (app, "settings");
     #if HAVE_SQLITE
     g_object_get (settings, "maximum-history-age", &max_history_age, NULL);
