@@ -82,14 +82,12 @@ struct _MidoriWebSettings
 
     gboolean remember_last_visited_pages;
     gint maximum_history_age;
-    gboolean remember_last_form_inputs;
     gboolean remember_last_downloaded_files;
 
     gchar* http_proxy;
     gboolean auto_detect_proxy;
     MidoriIdentity identify_as;
     gchar* ident_string;
-    gint cache_size;
 
     gint clear_private_data;
 };
@@ -150,6 +148,9 @@ enum
     PROP_OPEN_TABS_NEXT_TO_CURRENT,
     PROP_OPEN_POPUPS_IN_TABS,
 
+    PROP_AUTO_LOAD_IMAGES,
+    PROP_ENABLE_SCRIPTS,
+    PROP_ENABLE_PLUGINS,
     PROP_ZOOM_TEXT_AND_IMAGES,
     PROP_FIND_WHILE_TYPING,
     PROP_KINETIC_SCROLLING,
@@ -159,14 +160,12 @@ enum
 
     PROP_REMEMBER_LAST_VISITED_PAGES,
     PROP_MAXIMUM_HISTORY_AGE,
-    PROP_REMEMBER_LAST_FORM_INPUTS,
     PROP_REMEMBER_LAST_DOWNLOADED_FILES,
 
     PROP_HTTP_PROXY,
     PROP_AUTO_DETECT_PROXY,
     PROP_IDENTIFY_AS,
     PROP_IDENT_STRING,
-    PROP_CACHE_SIZE,
 
     PROP_CLEAR_PRIVATE_DATA
 };
@@ -215,6 +214,7 @@ midori_preferred_encoding_get_type (void)
         static const GEnumValue values[] = {
          { MIDORI_ENCODING_CHINESE, "MIDORI_ENCODING_CHINESE", N_("Chinese (BIG5)") },
          { MIDORI_ENCODING_JAPANESE, "MIDORI_ENCODING_JAPANESE", N_("Japanese (SHIFT_JIS)") },
+         { MIDORI_ENCODING_KOREAN, "MIDORI_ENCODING_KOREAN", N_("Korean (EUC-KR)") },
          { MIDORI_ENCODING_RUSSIAN, "MIDORI_ENCODING_RUSSIAN", N_("Russian (KOI8-R)") },
          { MIDORI_ENCODING_UNICODE, "MIDORI_ENCODING_UNICODE", N_("Unicode (UTF-8)") },
          { MIDORI_ENCODING_WESTERN, "MIDORI_ENCODING_WESTERN", N_("Western (ISO-8859-1)") },
@@ -320,7 +320,7 @@ midori_get_download_dir (void)
     const gchar* dir = g_get_user_special_dir (G_USER_DIRECTORY_DOWNLOAD);
     if (dir)
     {
-        g_mkdir_with_parents (dir, 0700);
+        katze_mkdir_with_parents (dir, 0700);
         return dir;
     }
     return g_get_home_dir ();
@@ -705,7 +705,11 @@ midori_web_settings_class_init (MidoriWebSettingsClass* class)
                                      "news-aggregator",
                                      _("News Aggregator"),
                                      _("An external news aggregator"),
+                                     #if HAVE_HILDON
+                                     "osso_notes",
+                                     #else
                                      NULL,
+                                     #endif
                                      flags));
 
     g_object_class_install_property (gobject_class,
@@ -753,7 +757,11 @@ midori_web_settings_class_init (MidoriWebSettingsClass* class)
                                      _("Open new pages in:"),
                                      _("Where to open new pages"),
                                      MIDORI_TYPE_NEW_PAGE,
+                                     #if HAVE_HILDON
+                                     MIDORI_NEW_PAGE_WINDOW,
+                                     #else
                                      MIDORI_NEW_PAGE_TAB,
+                                     #endif
                                      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
     g_object_class_install_property (gobject_class,
@@ -763,7 +771,11 @@ midori_web_settings_class_init (MidoriWebSettingsClass* class)
                                      _("Open external pages in:"),
                                      _("Where to open externally opened pages"),
                                      MIDORI_TYPE_NEW_PAGE,
+                                     #if HAVE_HILDON
+                                     MIDORI_NEW_PAGE_WINDOW,
+                                     #else
                                      MIDORI_NEW_PAGE_TAB,
+                                     #endif
                                      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
     g_object_class_install_property (gobject_class,
@@ -803,6 +815,32 @@ midori_web_settings_class_init (MidoriWebSettingsClass* class)
                                      G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 
 
+    /* Override properties to localize them for preference proxies */
+    g_object_class_install_property (gobject_class,
+                                     PROP_AUTO_LOAD_IMAGES,
+                                     g_param_spec_boolean (
+                                     "auto-load-images",
+                                     _("Load images automatically"),
+                                     _("Load and display images automatically"),
+                                     TRUE,
+                                     flags));
+    g_object_class_install_property (gobject_class,
+                                     PROP_ENABLE_SCRIPTS,
+                                     g_param_spec_boolean (
+                                     "enable-scripts",
+                                     _("Enable scripts"),
+                                     _("Enable embedded scripting languages"),
+                                     TRUE,
+                                     flags));
+    g_object_class_install_property (gobject_class,
+                                     PROP_ENABLE_PLUGINS,
+                                     g_param_spec_boolean (
+                                     "enable-plugins",
+                                     _("Enable Netscape plugins"),
+                                     _("Enable embedded Netscape plugin objects"),
+                                     TRUE,
+                                     flags));
+
     /**
      * MidoriWebSettings:zoom-text-and-images:
      *
@@ -899,15 +937,6 @@ midori_web_settings_class_init (MidoriWebSettingsClass* class)
                                      0, G_MAXINT, 30,
                                      flags));
 
-    g_object_class_install_property (gobject_class,
-                                     PROP_REMEMBER_LAST_FORM_INPUTS,
-                                     g_param_spec_boolean (
-                                     "remember-last-form-inputs",
-                                     _("Remember last form inputs"),
-                                     _("Whether the last form inputs are saved"),
-                                     TRUE,
-                                     G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
-
     g_object_class_install_property (gobject_class,
                                      PROP_REMEMBER_LAST_DOWNLOADED_FILES,
                                      g_param_spec_boolean (
@@ -978,15 +1007,6 @@ midori_web_settings_class_init (MidoriWebSettingsClass* class)
                                      NULL,
                                      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
-    g_object_class_install_property (gobject_class,
-                                     PROP_CACHE_SIZE,
-                                     g_param_spec_int (
-                                     "cache-size",
-                                     _("Cache size"),
-                                     _("The allowed size of the cache"),
-                                     0, G_MAXINT, 100,
-                                     G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
-
     /**
      * MidoriWebSettings:clear-private-data:
      *
@@ -1021,6 +1041,8 @@ notify_default_encoding_cb (GObject*    object,
         web_settings->preferred_encoding = MIDORI_ENCODING_CHINESE;
     else if (!strcmp (encoding, "SHIFT_JIS"))
         web_settings->preferred_encoding = MIDORI_ENCODING_JAPANESE;
+    else if (!strcmp (encoding, "EUC-KR"))
+        web_settings->preferred_encoding = MIDORI_ENCODING_KOREAN;
     else if (!strcmp (encoding, "KOI8-R"))
         web_settings->preferred_encoding = MIDORI_ENCODING_RUSSIAN;
     else if (!strcmp (encoding, "UTF-8"))
@@ -1041,7 +1063,6 @@ midori_web_settings_init (MidoriWebSettings* web_settings)
     web_settings->http_proxy = NULL;
     web_settings->show_panel_controls = TRUE;
     web_settings->open_popups_in_tabs = TRUE;
-    web_settings->remember_last_form_inputs = TRUE;
     web_settings->remember_last_downloaded_files = TRUE;
     web_settings->kinetic_scrolling = TRUE;
     web_settings->auto_detect_proxy = TRUE;
@@ -1277,6 +1298,9 @@ midori_web_settings_set_property (GObject*      object,
         case MIDORI_ENCODING_JAPANESE:
             g_object_set (object, "default-encoding", "SHIFT_JIS", NULL);
             break;
+       case MIDORI_ENCODING_KOREAN:
+            g_object_set (object, "default-encoding", "EUC-KR", NULL);
+            break;
         case MIDORI_ENCODING_RUSSIAN:
             g_object_set (object, "default-encoding", "KOI8-R", NULL);
             break;
@@ -1316,6 +1340,18 @@ midori_web_settings_set_property (GObject*      object,
         web_settings->open_popups_in_tabs = g_value_get_boolean (value);
         break;
 
+    case PROP_AUTO_LOAD_IMAGES:
+        g_object_set (web_settings, "WebKitWebSettings::auto-load-images",
+                      g_value_get_boolean (value), NULL);
+        break;
+    case PROP_ENABLE_SCRIPTS:
+        g_object_set (web_settings, "WebKitWebSettings::enable-scripts",
+                      g_value_get_boolean (value), NULL);
+        break;
+    case PROP_ENABLE_PLUGINS:
+        g_object_set (web_settings, "WebKitWebSettings::enable-plugins",
+                      g_value_get_boolean (value), NULL);
+        break;
     case PROP_ZOOM_TEXT_AND_IMAGES:
         web_settings->zoom_text_and_images = g_value_get_boolean (value);
         break;
@@ -1341,9 +1377,6 @@ midori_web_settings_set_property (GObject*      object,
     case PROP_MAXIMUM_HISTORY_AGE:
         web_settings->maximum_history_age = g_value_get_int (value);
         break;
-    case PROP_REMEMBER_LAST_FORM_INPUTS:
-        web_settings->remember_last_form_inputs = g_value_get_boolean (value);
-        break;
     case PROP_REMEMBER_LAST_DOWNLOADED_FILES:
         web_settings->remember_last_downloaded_files = g_value_get_boolean (value);
         break;
@@ -1375,9 +1408,6 @@ midori_web_settings_set_property (GObject*      object,
             #endif
         }
         break;
-    case PROP_CACHE_SIZE:
-        web_settings->cache_size = g_value_get_int (value);
-        break;
     case PROP_CLEAR_PRIVATE_DATA:
         web_settings->clear_private_data = g_value_get_int (value);
         break;
@@ -1522,6 +1552,18 @@ midori_web_settings_get_property (GObject*    object,
         g_value_set_boolean (value, web_settings->open_popups_in_tabs);
         break;
 
+    case PROP_AUTO_LOAD_IMAGES:
+        g_value_set_boolean (value, katze_object_get_boolean (web_settings,
+                             "WebKitWebSettings::auto-load-images"));
+        break;
+    case PROP_ENABLE_SCRIPTS:
+        g_value_set_boolean (value, katze_object_get_boolean (web_settings,
+                             "WebKitWebSettings::enable-scripts"));
+        break;
+    case PROP_ENABLE_PLUGINS:
+        g_value_set_boolean (value, katze_object_get_boolean (web_settings,
+                             "WebKitWebSettings::enable-plugins"));
+        break;
     case PROP_ZOOM_TEXT_AND_IMAGES:
         g_value_set_boolean (value, web_settings->zoom_text_and_images);
         break;
@@ -1547,9 +1589,6 @@ midori_web_settings_get_property (GObject*    object,
     case PROP_MAXIMUM_HISTORY_AGE:
         g_value_set_int (value, web_settings->maximum_history_age);
         break;
-    case PROP_REMEMBER_LAST_FORM_INPUTS:
-        g_value_set_boolean (value, web_settings->remember_last_form_inputs);
-        break;
     case PROP_REMEMBER_LAST_DOWNLOADED_FILES:
         g_value_set_boolean (value, web_settings->remember_last_downloaded_files);
         break;
@@ -1571,9 +1610,6 @@ midori_web_settings_get_property (GObject*    object,
         }
         g_value_set_string (value, web_settings->ident_string);
         break;
-    case PROP_CACHE_SIZE:
-        g_value_set_int (value, web_settings->cache_size);
-        break;
     case PROP_CLEAR_PRIVATE_DATA:
         g_value_set_int (value, web_settings->clear_private_data);
         break;
