@@ -11,14 +11,14 @@
 */
 
 #include "sokoke.h"
-#include "midori-stock.h"
-
-#include "compat.h"
 
 #if HAVE_CONFIG_H
     #include <config.h>
 #endif
 
+#include "compat.h"
+#include "midori-stock.h"
+
 #if HAVE_UNISTD_H
     #include <unistd.h>
 #endif
@@ -73,7 +73,8 @@ sokoke_js_script_eval (JSContextRef js_context,
     {
         JSStringRef js_message = JSValueToStringCopy (js_context,
                                                       js_exception, NULL);
-        *exception = sokoke_js_string_utf8 (js_message);
+        if (exception)
+            *exception = sokoke_js_string_utf8 (js_message);
         JSStringRelease (js_message);
         js_value = JSValueMakeNull (js_context);
     }
@@ -146,42 +147,32 @@ sokoke_spawn_program (const gchar* command,
                       const gchar* argument,
                       gboolean     quote)
 {
-    gchar* argument_escaped;
-    gchar* command_ready;
-    gchar** argv;
+    GAppInfo* info;
+    GFile* file;
+    GList* files;
     GError* error;
 
     g_return_val_if_fail (command != NULL, FALSE);
     g_return_val_if_fail (argument != NULL, FALSE);
+    /* quote is ignored */
 
-    argument_escaped = quote ? g_shell_quote (argument) : g_strdup (argument);
-    if (strstr (command, "%s"))
-        command_ready = g_strdup_printf (command, argument_escaped);
-    else
-        command_ready = g_strconcat (command, " ", argument_escaped, NULL);
+    info = g_app_info_create_from_commandline (command,
+        NULL, G_APP_INFO_CREATE_NONE, NULL);
+    file = g_file_new_for_commandline_arg (argument);
+    files = g_list_append (NULL, file);
 
     error = NULL;
-    if (!g_shell_parse_argv (command_ready, NULL, &argv, &error))
+    if (!g_app_info_launch (info, files, NULL, &error))
     {
         error_dialog (_("Could not run external program."), error->message);
         g_error_free (error);
-        g_free (command_ready);
-        g_free (argument_escaped);
+        g_object_unref (file);
+        g_list_free (files);
         return FALSE;
     }
 
-    error = NULL;
-    if (!g_spawn_async (NULL, argv, NULL,
-        (GSpawnFlags)G_SPAWN_SEARCH_PATH | G_SPAWN_DO_NOT_REAP_CHILD,
-        NULL, NULL, NULL, &error))
-    {
-        error_dialog (_("Could not run external program."), error->message);
-        g_error_free (error);
-    }
-
-    g_strfreev (argv);
-    g_free (command_ready);
-    g_free (argument_escaped);
+    g_object_unref (file);
+    g_list_free (files);
     return TRUE;
 }
 
@@ -608,24 +599,6 @@ sokoke_xfce_header_new (const gchar* icon,
     return NULL;
 }
 
-GtkWidget*
-sokoke_hig_frame_new (const gchar* title)
-{
-    /* Create a frame with no actual frame but a bold label and indentation */
-    GtkWidget* frame = gtk_frame_new (NULL);
-    #ifdef G_OS_WIN32
-    gtk_frame_set_label (GTK_FRAME (frame), title);
-    #else
-    gchar* title_bold = g_strdup_printf ("<b>%s</b>", title);
-    GtkWidget* label = gtk_label_new (NULL);
-    gtk_label_set_markup (GTK_LABEL (label), title_bold);
-    g_free (title_bold);
-    gtk_frame_set_label_widget (GTK_FRAME (frame), label);
-    gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_NONE);
-    #endif
-    return frame;
-}
-
 void
 sokoke_widget_set_pango_font_style (GtkWidget* widget,
                                     PangoStyle style)
@@ -934,6 +907,11 @@ sokoke_time_t_to_julian (const time_t* timestamp)
 void
 sokoke_register_stock_items (void)
 {
+    GtkIconSource* icon_source;
+    GtkIconSet* icon_set;
+    GtkIconFactory* factory;
+    gsize i;
+
     typedef struct
     {
         const gchar* stock_id;
@@ -942,11 +920,6 @@ sokoke_register_stock_items (void)
         guint keyval;
         const gchar* fallback;
     } FatStockItem;
-    GtkIconSource* icon_source;
-    GtkIconSet* icon_set;
-    GtkIconFactory* factory = gtk_icon_factory_new ();
-    gsize i;
-
     static FatStockItem items[] =
     {
         { STOCK_EXTENSION, NULL, 0, 0, GTK_STOCK_CONVERT },
@@ -958,7 +931,7 @@ sokoke_register_stock_items (void)
         { STOCK_TRANSFER, NULL, 0, 0, GTK_STOCK_SAVE },
 
         { STOCK_BOOKMARK,       N_("_Bookmark"), 0, 0, GTK_STOCK_FILE },
-        { STOCK_BOOKMARKS,      N_("_Bookmarks"), 0, 0, GTK_STOCK_DIRECTORY },
+        { STOCK_BOOKMARKS,      N_("_Bookmarks"), GDK_CONTROL_MASK, GDK_B, GTK_STOCK_DIRECTORY },
         { STOCK_BOOKMARK_ADD,   N_("Add Boo_kmark"), 0, 0, GTK_STOCK_ADD },
         { STOCK_CONSOLE,        N_("_Console"), 0, 0, GTK_STOCK_DIALOG_WARNING },
         { STOCK_EXTENSIONS,     N_("_Extensions"), 0, 0, GTK_STOCK_CONVERT },
@@ -972,6 +945,7 @@ sokoke_register_stock_items (void)
         { STOCK_WINDOW_NEW,     N_("New _Window"), 0, 0, GTK_STOCK_ADD },
     };
 
+    factory = gtk_icon_factory_new ();
     for (i = 0; i < G_N_ELEMENTS (items); i++)
     {
         icon_set = gtk_icon_set_new ();
@@ -991,6 +965,55 @@ sokoke_register_stock_items (void)
     gtk_stock_add ((GtkStockItem*)items, G_N_ELEMENTS (items));
     gtk_icon_factory_add_default (factory);
     g_object_unref (factory);
+
+    #if HAVE_HILDON
+    /* Maemo doesn't theme stock icons. So we map platform icons
+        to stock icons. These are all monochrome toolbar icons. */
+    typedef struct
+    {
+        const gchar* stock_id;
+        const gchar* icon_name;
+    } CompatItem;
+    static CompatItem compat_items[] =
+    {
+        { GTK_STOCK_ADD,        "general_add" },
+        { GTK_STOCK_BOLD,       "general_bold" },
+        { GTK_STOCK_CLOSE,      "general_close_b" },
+        { GTK_STOCK_DELETE,     "general_delete" },
+        { GTK_STOCK_DIRECTORY,  "general_toolbar_folder" },
+        { GTK_STOCK_FIND,       "general_search" },
+        { GTK_STOCK_FULLSCREEN, "general_fullsize_b" },
+        { GTK_STOCK_GO_BACK,    "general_back" },
+        { GTK_STOCK_GO_FORWARD, "general_forward" },
+        { GTK_STOCK_GO_UP,      "filemanager_folder_up" },
+        { GTK_STOCK_GOTO_FIRST, "pdf_viewer_first_page" },
+        { GTK_STOCK_GOTO_LAST,  "pdf_viewer_last_page" },
+        { GTK_STOCK_INFO,       "general_information" },
+        { GTK_STOCK_ITALIC,     "general_italic" },
+        { GTK_STOCK_JUMP_TO,    "general_move_to_folder" },
+        { GTK_STOCK_PREFERENCES,"general_settings" },
+        { GTK_STOCK_REFRESH,    "general_refresh" },
+        { GTK_STOCK_SAVE,       "notes_save" },
+        { GTK_STOCK_STOP,       "general_stop" },
+        { GTK_STOCK_UNDERLINE,  "notes_underline" },
+        { GTK_STOCK_ZOOM_IN,    "pdf_zoomin" },
+        { GTK_STOCK_ZOOM_OUT,   "pdf_zoomout" },
+    };
+
+    factory = gtk_icon_factory_new ();
+    for (i = 0; i < G_N_ELEMENTS (compat_items); i++)
+    {
+        icon_set = gtk_icon_set_new ();
+        icon_source = gtk_icon_source_new ();
+        gtk_icon_source_set_icon_name (icon_source, compat_items[i].icon_name);
+        gtk_icon_set_add_source (icon_set, icon_source);
+        gtk_icon_source_free (icon_source);
+        gtk_icon_factory_add (factory, compat_items[i].stock_id, icon_set);
+        gtk_icon_set_unref (icon_set);
+    }
+    gtk_icon_factory_add_default (factory);
+    g_object_unref (factory);
+    #endif
 }
 
 /**
@@ -1234,3 +1257,41 @@ sokoke_replace_variables (const gchar* template,
 
     return result;
 }
+
+/**
+ * sokoke_window_activate_key:
+ * @window: a #GtkWindow
+ * @event: a #GdkEventKey
+ *
+ * Attempts to activate they key from the event, much
+ * like gtk_window_activate_key(), including keys
+ * that gtk_accelerator_valid() considers invalid.
+ *
+ * Return value: %TRUE on success
+ **/
+gboolean
+sokoke_window_activate_key (GtkWindow*   window,
+                            GdkEventKey* event)
+{
+    gchar *accel_name;
+    GQuark accel_quark;
+    GObject* object;
+    GSList *slist;
+
+    if (gtk_window_activate_key (window, event))
+        return TRUE;
+
+    /* We don't use gtk_accel_groups_activate because it refuses to
+        activate anything that gtk_accelerator_valid doesn't like. */
+    accel_name = gtk_accelerator_name (event->keyval, (event->state & gtk_accelerator_get_default_mod_mask ()));
+    accel_quark = g_quark_from_string (accel_name);
+    g_free (accel_name);
+    object = G_OBJECT (window);
+
+    for (slist = gtk_accel_groups_from_object (object); slist; slist = slist->next)
+        if (gtk_accel_group_activate (slist->data, accel_quark,
+                                      object, event->keyval, event->state))
+            return TRUE;
+
+    return FALSE;
+}
